package com.nbclass.system.service;

import java.io.Serializable;
import java.util.List;

import com.nbclass.activity.model.Application;
import com.nbclass.system.model.User;
import com.nbclass.vo.UserOnlineVo;
import com.nbclass.vo.base.ResponseVo;
/**
 * @version V1.0
 * @date 2018年7月11日
 * @author superzheng
 */
public interface UserService {

    /**
     * 根据用户名查询用户
     * @param username
     * @return user
     */
    User selectByUsername(String username);

    /**
     * 注册用户
     * @param user
     * @return int
     */
    int register(User user);

    /**
     * 更新最后登录时间
     * @param user
     */
    void updateLastLoginTime(User user);

    /**
     * 根据条件查询用户列表
     * @param user
     * @return list
     */
    List<User> selectUsers(User user);

    /**
     * 根据用户id查询用户
     * @param userId
     * @return user
     */
    User selectByUserId(String userId);

    /**
     * 根据用户id更新用户信息
     * @param user
     * @return int
     */
    int updateByUserId(User user);

    /**
     * 根据用户id集合批量更新用户状态
     * @param userIds
     * @param status
     * @return int
     */
    int updateStatusBatch(List<String> userIds, Integer status);

    /**
     * 根据用户id分配角色集合
     * @param userId
     * @param roleIds
     * @return int
     */
    ResponseVo<Object> addAssignRole(String userId, List<String> roleIds);

    /**
     * 根据主键更新用户信息
     * @param user
     * @return int
     */
    int updateUserByPrimaryKey(User user);

    /**
     * 查询在线用户
     * @param userOnlineVo
     * @return list
     */
    List<UserOnlineVo> selectOnlineUsers(UserOnlineVo userOnlineVo);

    /**
     * 踢出用户
     * @param sessionId 会话id
     * @param username 用户名
     */
    void kickout(Serializable sessionId, String username);

    /**
     * 修改企业微信用户信息
     * @param user
     */
    public void updateQyWxUser(User user);

    /**
     * 查询用户的职位信息
     * @return
     */
    String getUserRole(String userId);
}
